This document describes changes to the Apache HTTPD API from version 2.2
to 2.4, that may be of interest to module/application developers and
core hacks. As of the first GA release of the 2.4 branch API
compatibility is preserved for the life of the 2.4 branch. (The

VERSIONING

description for the 2.4 release provides more information about API
compatibility.)

API changes fall into two categories: APIs that are altogether new, and
existing APIs that are expanded or changed. The latter are further
divided into those where all changes are backwards-compatible (so
existing modules can ignore them), and those that might require
attention by maintainers. As with the transition from HTTPD 2.0 to 2.2,
existing modules and applications will require recompiling and may call
for some attention, but most should not require any substantial updating
(although some may be able to take advantage of API changes to offer
significant improvements).

For the purpose of this document, the API is split according to the
public header files. These headers are themselves the reference
documentation, and can be used to generate a browsable HTML reference
with

make docs

.

Changed APIs {#api_changes}
============

ap\_expr (NEW!) {#ap_expr}
---------------

Introduces a new API to parse and evaluate boolean and algebraic
expressions, including provision for a standard syntax and customised
variants.

ap\_listen (changed; backwards-compatible) {#ap_listen}
------------------------------------------

Introduces a new API to enable httpd child processes to serve different
purposes.

ap\_mpm (changed) {#ap_mpm}
-----------------

ap\_mpm\_run

is replaced by a new

mpm

hook. Also

ap\_graceful\_stop\_signalled

is lost, and

ap\_mpm\_register\_timed\_callback

is new.

ap\_regex (changed) {#ap_regex}
-------------------

In addition to the existing regexp wrapper, a new higher-level API

ap\_rxplus

is now provided. This provides the capability to compile Perl-style
expressions like

s/regexp/replacement/flags

and to execute them against arbitrary strings. Support for regexp
backreferences is also added.

ap\_slotmem (NEW!) {#ap_slotmem}
------------------

Introduces an API for modules to allocate and manage memory slots, most
commonly for shared memory.

ap\_socache (NEW!) {#ap_socache}
------------------

API to manage a shared object cache.

heartbeat (NEW!) {#heartbeat}
----------------

common structures for heartbeat modules

ap\_parse\_htaccess (changed) {#ap_parse_htaccess}
-----------------------------

The function signature for

ap\_parse\_htaccess

has been changed. A

apr\_table\_t

of individual directives allowed for override must now be passed
(override remains).

http\_config (changed) {#http_config}
----------------------

Introduces per-module, per-directory loglevels, including macro
wrappers.

New

AP\_DECLARE\_MODULE

macro to declare all modules.

New

APLOG\_USE\_MODULE

macro necessary for per-module loglevels in multi-file modules.

New API to retain data across module unload/load

New

check\_config

hook

New

ap\_process\_fnmatch\_configs()

function to process wildcards

Change

ap\_configfile\_t

,

ap\_cfg\_getline()

,

ap\_cfg\_getc()

to return error codes, and add

ap\_pcfg\_strerror()

for retrieving an error description.

Any config directive permitted in ACCESS\_CONF context must now
correctly handle being called from an .htaccess file via the new

AllowOverrideList

directive. ap\_check\_cmd\_context() accepts a new flag
NOT\_IN\_HTACCESS to detect this case.

http\_core (changed) {#http_core}
--------------------

REMOVED

ap\_default\_type

,

ap\_requires

, all 2.2 authnz API

Introduces Optional Functions for logio and authnz

New function

ap\_get\_server\_name\_for\_url

to support IPv6 literals.

New function

ap\_register\_errorlog\_handler

to register error log format string handlers.

Arguments of

error\_log

hook have changed. Declaration has moved to

http\_core.h

.

New function

ap\_state\_query

to determine if the server is in the initial configuration preflight
phase or not. This is both easier to use and more correct than the old
method of creating a pool userdata entry in the process pool.

New function

ap\_get\_conn\_socket

to get the socket descriptor for a connection. This should be used
instead of accessing the core connection config directly.

httpd (changed) {#httpd}
---------------

Introduce per-directory, per-module loglevel

New loglevels

APLOG\_TRACEn

Introduce errorlog ids for requests and connections

Support for mod\_request kept\_body

Support buffering filter data for async requests

New

CONN\_STATE

values

Function changes:

ap\_escape\_html

updated;

ap\_unescape\_all

,

ap\_escape\_path\_segment\_buffer

Modules that load other modules later than the

EXEC\_ON\_READ

config reading stage need to call

ap\_reserve\_module\_slots()

or

ap\_reserve\_module\_slots\_directive()

in their

pre\_config hook

.

The useragent IP address per request can now be tracked independently of
the client IP address of the connection, for support of deployments with
load balancers.

http\_log (changed) {#http_log}
-------------------

Introduce per-directory, per-module loglevel

New loglevels

APLOG\_TRACEn

ap\_log\_\*error

become macro wrappers (backwards-compatible if

APLOG\_MARK

macro is used, except that is no longer possible to use

\#ifdef

inside the argument list)

piped logging revamped

module\_index

added to error\_log hook

new function:

ap\_log\_command\_line

http\_request (changed) {#http_request}
-----------------------

New auth\_internal API and auth\_provider API

New

EOR

bucket type

New function

ap\_process\_async\_request

New flags

AP\_AUTH\_INTERNAL\_PER\_CONF

and

AP\_AUTH\_INTERNAL\_PER\_URI

New

access\_checker\_ex

hook to apply additional access control and/or bypass authentication.

New functions

ap\_hook\_check\_access\_ex

,

ap\_hook\_check\_access

,

ap\_hook\_check\_authn

,

ap\_hook\_check\_authz

which accept

AP\_AUTH\_INTERNAL\_PER\_\*

flags

DEPRECATED direct use of

ap\_hook\_access\_checker

,

access\_checker\_ex

,

ap\_hook\_check\_user\_id

,

ap\_hook\_auth\_checker

When possible, registering all access control hooks (including
authentication and authorization hooks) using

AP\_AUTH\_INTERNAL\_PER\_CONF

is recommended. If all modules\' access control hooks are registered
with this flag, then whenever the server handles an internal sub-request
that matches the same set of access control configuration directives as
the initial request (which is the common case), it can avoid invoking
the access control hooks another time.

If your module requires the old behavior and must perform access control
checks on every sub-request with a different URI from the initial
request, even if that URI matches the same set of access control
configuration directives, then use

AP\_AUTH\_INTERNAL\_PER\_URI

.

mod\_auth (NEW!) {#mod_auth}
----------------

Introduces the new provider framework for authn and authz

mod\_cache (changed) {#mod_cache}
--------------------

Introduces a

commit\_entity()

function to the cache provider interface, allowing atomic writes to
cache. Add a

cache\_status()

hook to report the cache decision. All private structures and functions
were removed.

mod\_core (NEW!) {#mod_core}
----------------

This introduces low-level APIs to send arbitrary headers, and exposes
functions to handle HTTP OPTIONS and TRACE.

mod\_cache\_disk (changed) {#mod_cache_disk}
--------------------------

Changes the disk format of the disk cache to support atomic cache
updates without locking. The device/inode pair of the body file is
embedded in the header file, allowing confirmation that the header and
body belong to one another.

mod\_disk\_cache (renamed) {#mod_disk_cache}
--------------------------

The mod\_disk\_cache module has been renamed to mod\_cache\_disk in
order to be consistent with the naming of other modules within the
server.

mod\_request (NEW!) {#mod_request}
-------------------

The API for

mod\_request

, to make input data available to multiple application/handler modules
where required, and to parse HTML form data.

mpm\_common (changed) {#mpm_common}
---------------------

REMOVES:

accept

,

lockfile

,

lock\_mech

,

set\_scoreboard

(locking uses the new ap\_mutex API)

NEW API to drop privileges (delegates this platform-dependent function
to modules)

NEW Hooks:

mpm\_query

,

timed\_callback

, and

get\_name

CHANGED interfaces:

monitor

hook,

ap\_reclaim\_child\_processes

,

ap\_relieve\_child\_processes

scoreboard (changed) {#scoreboard}
--------------------

ap\_get\_scoreboard\_worker

is made non-backwards-compatible as an alternative version is
introduced. Additional proxy\_balancer support. Child status stuff
revamped.

util\_cookies (NEW!) {#util_cookies}
--------------------

Introduces a new API for managing HTTP Cookies.

util\_ldap (changed) {#util_ldap}
--------------------

no description available

util\_mutex (NEW!) {#util_mutex}
------------------

A wrapper for APR proc and global mutexes in httpd, providing common
configuration for the underlying mechanism and location of lock files.

util\_script (changed) {#util_script}
----------------------

NEW:

ap\_args\_to\_table

util\_time (changed) {#util_time}
--------------------

NEW:

ap\_recent\_ctime\_ex

Specific information on upgrading modules from 2.2 {#upgrading}
==================================================

Logging {#upgrading_logging}
-------

In order to take advantage of per-module loglevel configuration, any
source file that calls the

ap\_log\_\*

functions should declare which module it belongs to. If the module\'s
module\_struct is called

foo\_module

, the following code can be used to remain backward compatible with
HTTPD 2.0 and 2.2:

\#include \<http\_log.h\>

\#ifdef APLOG\_USE\_MODULE

APLOG\_USE\_MODULE(foo);

\#endif

Note: This is absolutely required for C++-language modules. It can be
skipped for C-language modules, though that breaks module-specific log
level support for files without it.

The number of parameters of the

ap\_log\_\*

functions and the definition of

APLOG\_MARK

has changed. Normally, the change is completely transparent. However,
changes are required if a module uses

APLOG\_MARK

as a parameter to its own functions or if a module calls

ap\_log\_\*

without passing

APLOG\_MARK

. A module which uses wrappers around

ap\_log\_\*

typically uses both of these constructs.

The easiest way to change code which passes

APLOG\_MARK

to its own functions is to define and use a different macro that expands
to the parameters required by those functions, as

APLOG\_MARK

should only be used when calling

ap\_log\_\*

directly. In this way, the code will remain compatible with HTTPD 2.0
and 2.2.

Code which calls

ap\_log\_\*

without passing

APLOG\_MARK

will necessarily differ between 2.4 and earlier releases, as 2.4
requires a new third argument,

APLOG\_MODULE\_INDEX

.

/\* code for httpd 2.0/2.2 \*/

ap\_log\_perror(file, line, APLOG\_ERR, 0, p, \"Failed to allocate
dynamic lock structure\");

/\* code for httpd 2.4 \*/

ap\_log\_perror(file, line, APLOG\_MODULE\_INDEX, APLOG\_ERR, 0, p,
\"Failed to allocate dynamic lock structure\");

ap\_log\_\*error

are now implemented as macros. This means that it is no longer possible
to use

\#ifdef

inside the argument list of

ap\_log\_\*error

, as this would cause undefined behavior according to C99.

A

server\_rec

pointer must be passed to

ap\_log\_error()

when called after startup. This was always appropriate, but there are
even more limitations with a

NULL

server\_rec

in 2.4 than in previous releases. Beginning with 2.3.12, the global
variable

ap\_server\_conf

can always be used as the

server\_rec

parameter, as it will be

NULL

only when it is valid to pass

NULL

to

ap\_log\_error()

.

ap\_server\_conf

should be used only when a more appropriate

server\_rec

is not available.

Consider the following changes to take advantage of the new

APLOG\_TRACE1..8

log levels:

Check current use of

APLOG\_DEBUG

and consider if one of the

APLOG\_TRACEn

levels is more appropriate.

If your module currently has a mechanism for configuring the amount of
debug logging which is performed, consider eliminating that mechanism
and relying on the use of different

APLOG\_TRACEn

levels. If expensive trace processing needs to be bypassed depending on
the configured log level, use the

APLOGtrace

n

and

APLOGrtrace

n

macros to first check if tracing is enabled.

Modules sometimes add process id and/or thread id to their log messages.
These ids are now logged by default, so it may not be necessary for the
module to log them explicitly. (Users may remove them from the error log
format, but they can be instructed to add it back if necessary for
problem diagnosis.)

If your module uses these existing APIs\... {#upgrading_byfunction}
-------------------------------------------

ap\_default\_type()

This is no longer available; Content-Type must be configured explicitly
or added by the application.

ap\_get\_server\_name()

If the returned server name is used in a URL, use

ap\_get\_server\_name\_for\_url()

instead. This new function handles the odd case where the server name is
an IPv6 literal address.

ap\_get\_server\_version()

For logging purposes, where detailed information is appropriate, use

ap\_get\_server\_description()

. When generating output, where the amount of information should be
configurable by ServerTokens, use

ap\_get\_server\_banner()

.

ap\_graceful\_stop\_signalled()

Replace with a call to

ap\_mpm\_query(AP\_MPMQ\_MPM\_STATE)

and checking for state

AP\_MPMQ\_STOPPING

.

ap\_max\_daemons\_limit

,

ap\_my\_generation

, and

ap\_threads\_per\_child

Use

ap\_mpm\_query()

query codes

AP\_MPMQ\_MAX\_DAEMON\_USED

,

AP\_MPMQ\_GENERATION

, and

AP\_MPMQ\_MAX\_THREADS

, respectively.

ap\_mpm\_query()

Ensure that it is not used until after the register-hooks hook has
completed. Otherwise, an MPM built as a DSO would not have had a chance
to enable support for this function.

ap\_requires()

The core server now provides better infrastructure for handling

Require

configuration. Register an auth provider function for each supported
entity using

ap\_register\_auth\_provider()

. The function will be called as necessary during

Require

processing. (Consult bundled modules for detailed examples.)

ap\_server\_conf-\>process-\>pool

userdata

Optional:

If your module uses this to determine which pass of the startup hooks is
being run, use

ap\_state\_query(AP\_SQ\_MAIN\_STATE)

.

If your module uses this to maintain data across the unloading and
reloading of your module, use

ap\_retained\_data\_create()

and

ap\_retained\_data\_get()

.

apr\_global\_mutex\_create()

,

apr\_proc\_mutex\_create()

Optional: See

ap\_mutex\_register()

,

ap\_global\_mutex\_create()

, and

ap\_proc\_mutex\_create()

; these allow your mutexes to be configurable with the

Mutex

directive; you can also remove any configuration mechanisms in your
module for such mutexes

CORE\_PRIVATE

This is now unnecessary and ignored.

dav\_new\_error()

and

dav\_new\_error\_tag()

Previously, these assumed that

errno

contained information describing the failure. Now, an

apr\_status\_t

parameter must be provided. Pass 0/APR\_SUCCESS if there is no such
error information, or a valid

apr\_status\_t

value otherwise.

mpm\_default.h

,

DEFAULT\_LOCKFILE

,

DEFAULT\_THREAD\_LIMIT

,

DEFAULT\_PIDLOG

, etc.

The header file and most of the default configuration values set in it
are no longer visible to modules. (Most can still be overridden at build
time.)

DEFAULT\_PIDLOG

and

DEFAULT\_REL\_RUNTIMEDIR

are now universally available via

ap\_config.h

.

unixd\_config

This has been renamed to ap\_unixd\_config.

conn\_rec-\>remote\_ip

and

conn\_rec-\>remote\_addr

These fields have been renamed in order to distinguish between the
client IP address of the connection and the useragent IP address of the
request (potentially overridden by a load balancer or proxy). References
to either of these fields must be updated with one of the following
options, as appropriate for the module:

When you require the IP address of the user agent, which might be
connected directly to the server, or might optionally be separated from
the server by a transparent load balancer or proxy, use

request\_rec-\>useragent\_ip

and

request\_rec-\>useragent\_addr

.

When you require the IP address of the client that is connected directly
to the server, which might be the useragent or might be the load
balancer or proxy itself, use

conn\_rec-\>client\_ip

and

conn\_rec-\>client\_addr

.

If your module interfaces with this feature\... {#upgrading_byfeature}
-----------------------------------------------

suEXEC

Optional: If your module logs an error when

ap\_unixd\_config.suexec\_enabled

is 0, also log the value of the new field

suexec\_disabled\_reason

, which contains an explanation of why it is not available.

Extended status data in the scoreboard

In previous releases,

ExtendedStatus

had to be set to

On

, which in turn required that mod\_status was loaded. In 2.4, just set

ap\_extended\_status

to

1

in a pre-config hook and the extended status data will be available.

Does your module\... {#upgrading_newfeatures}
--------------------

Parse query args

Consider if

ap\_args\_to\_table()

would be helpful.

Parse form data\...

Use

ap\_parse\_form\_data()

.

Check for request header fields

Content-Length

and

Transfer-Encoding

to see if a body was specified

Use

ap\_request\_has\_body()

.

Implement cleanups which clear pointer variables

Use

ap\_pool\_cleanup\_set\_null()

.

Create run-time files such as shared memory files, pid files, etc.

Use

ap\_runtime\_dir\_relative()

so that the global configuration for the location of such files, either
by the

DEFAULT\_REL\_RUNTIMEDIR

compile setting or the

DefaultRuntimeDir

directive, will be respected.

Apache httpd 2.4.2 and above.
