mod\_authz\_dbm

Group authorization using DBM files

Extension

mod\_authz\_dbm.c

authz\_dbm\_module

This module provides authorization capabilities so that authenticated
users can be allowed or denied access to portions of the web site by
group membership. Similar functionality is provided by

mod\_authz\_groupfile

.

Require

The Require Directives {#requiredirectives}
======================

Apache\'s

Require

directives are used during the authorization phase to ensure that a user
is allowed to access a resource. mod\_authz\_dbm extends the
authorization types with

dbm-group

.

Since v2.4.8,

expressions

are supported within the DBM require directives.

Require dbm-group {#reqgroup}
-----------------

This directive specifies group membership that is required for the user
to gain access.

Require dbm-group admin

Require dbm-file-group {#reqfilegroup}
----------------------

When this directive is specified, the user must be a member of the group
assigned to the file being accessed.

Require dbm-file-group

Example usage {#examples}
=============

Note that using mod\_authz\_dbm requires you to require

dbm-group

instead of

group

:

\<Directory \"/foo/bar\"\> AuthType Basic AuthName \"Secure Area\"
AuthBasicProvider dbm AuthDBMUserFile \"site/data/users\"
AuthDBMGroupFile \"site/data/users\" Require dbm-group admin
\</Directory\>

AuthDBMGroupFile

Sets the name of the database file containing the list of user groups
for authorization

AuthDBMGroupFile

file-path

directory

.htaccess

AuthConfig

The

AuthDBMGroupFile

directive sets the name of a DBM file containing the list of user groups
for user authorization.

File-path

is the absolute path to the group file.

The group file is keyed on the username. The value for a user is a
comma-separated list of the groups to which the users belongs. There
must be no whitespace within the value, and it must never contain any
colons.

> **Note**
>
> Make sure that the
>
> AuthDBMGroupFile
>
> is stored outside the document tree of the web-server. Do
>
> not
>
> put it in the directory that it protects. Otherwise, clients will be
> able to download the
>
> AuthDBMGroupFile
>
> unless otherwise protected.

Combining Group and Password DBM files: In some cases it is easier to
manage a single database which contains both the password and group
details for each user. This simplifies any support programs that need to
be written: they now only have to deal with writing to and locking a
single DBM file. This can be accomplished by first setting the group and
password files to point to the same DBM:

AuthDBMGroupFile \"/www/userbase\" AuthDBMUserFile \"/www/userbase\"

The key for the single DBM is the username. The value consists of

Encrypted Password : List of Groups \[ : (ignored) \]

The password section contains the encrypted password as before. This is
followed by a colon and the comma separated list of groups. Other data
may optionally be left in the DBM file after another colon; it is
ignored by the authorization module.

AuthzDBMType

Sets the type of database file that is used to store list of user groups

AuthzDBMType default\|SDBM\|GDBM\|NDBM\|DB

AuthzDBMType default

directory

.htaccess

AuthConfig

Sets the type of database file that is used to store the list of user
groups. The default database type is determined at compile time. The
availability of other types of database files also depends on

compile-time settings

.

It is crucial that whatever program you use to create your group files
is configured to use the same type of database.
