mod\_setenvif

Allows the setting of environment variables based on characteristics of
the request

Base

mod\_setenvif.c

setenvif\_module

The

mod\_setenvif

module allows you to set internal environment variables according to
whether different aspects of the request match regular expressions you
specify. These environment variables can be used by other parts of the
server to make decisions about actions to be taken, as well as becoming
available to CGI scripts and SSI pages.

The directives are considered in the order they appear in the
configuration files. So more complex sequences can be used, such as this
example, which sets

netscape

if the browser is mozilla but not MSIE.

BrowserMatch \^Mozilla netscape BrowserMatch MSIE !netscape

When the server looks up a path via an internal

subrequest

such as looking for a

DirectoryIndex

or generating a directory listing with

mod\_autoindex

, per-request environment variables are

not

inherited in the subrequest. Additionally,

SetEnvIf

directives are not separately evaluated in the subrequest due to the API
phases

mod\_setenvif

takes action in.

Environment Variables in Apache HTTP Server

BrowserMatch

Sets environment variables conditional on HTTP User-Agent

BrowserMatch

regex \[!\]env-variable

\[=

value

\] \[\[!\]

env-variable

\[=

value

\]\] \...

server config

virtual host

directory

.htaccess

FileInfo

The

BrowserMatch

is a special cases of the

SetEnvIf

directive that sets environment variables conditional on the

User-Agent

HTTP request header. The following two lines have the same effect:

BrowserMatch Robot is\_a\_robot SetEnvIf User-Agent Robot is\_a\_robot

Some additional examples:

BrowserMatch \^Mozilla forms jpeg=yes browser=netscape BrowserMatch
\"\^Mozilla/\[2-3\]\" tables agif frames javascript BrowserMatch MSIE
!javascript

BrowserMatchNoCase

Sets environment variables conditional on User-Agent without respect to
case

BrowserMatchNoCase

regex \[!\]env-variable

\[=

value

\] \[\[!\]

env-variable

\[=

value

\]\] \...

server config

virtual host

directory

.htaccess

FileInfo

The

BrowserMatchNoCase

directive is semantically identical to the

BrowserMatch

directive. However, it provides for case-insensitive matching. For
example:

BrowserMatchNoCase mac platform=macintosh BrowserMatchNoCase win
platform=windows

The

BrowserMatch

and

BrowserMatchNoCase

directives are special cases of the

SetEnvIf

and

SetEnvIfNoCase

directives. The following two lines have the same effect:

BrowserMatchNoCase Robot is\_a\_robot SetEnvIfNoCase User-Agent Robot
is\_a\_robot

SetEnvIf

Sets environment variables based on attributes of the request

SetEnvIf

attribute regex \[!\]env-variable

\[=

value

\] \[\[!\]

env-variable

\[=

value

\]\] \...

server config

virtual host

directory

.htaccess

FileInfo

The

SetEnvIf

directive defines environment variables based on attributes of the
request. The

attribute

specified in the first argument can be one of the following things:

An HTTP request header field (see

RFC2616

for more information about these); for example:

Host

,

User-Agent

,

Referer

, and

Accept-Language

. A regular expression may be used to specify a set of request headers.

One of the following aspects of the request:

Remote\_Host

\- the hostname (if available) of the client making the request

Remote\_Addr

\- the IP address of the client making the request

Server\_Addr

\- the IP address of the server on which the request was received (only
with versions later than 2.0.43)

Request\_Method

\- the name of the method being used (

GET

,

POST

,

et cetera

)

Request\_Protocol

\- the name and version of the protocol with which the request was made
(

e.g.

, \"HTTP/0.9\", \"HTTP/1.1\",

etc.

)

Request\_URI

\- the resource requested on the HTTP request line \-- generally the
portion of the URL following the scheme and host portion without the
query string. See the

RewriteCond

directive of

mod\_rewrite

for extra information on how to match your query string.

The name of an environment variable in the list of those associated with
the request. This allows

SetEnvIf

directives to test against the result of prior matches. Only those
environment variables defined by earlier

SetEnvIf\[NoCase\]

directives are available for testing in this manner. \'Earlier\' means
that they were defined at a broader scope (such as server-wide) or
previously in the current directive\'s scope. Environment variables will
be considered only if there was no match among request characteristics
and a regular expression was not used for the

attribute

.

The second argument (

regex

) is a

regular expression

. If the

regex

matches against the

attribute

, then the remainder of the arguments are evaluated.

The rest of the arguments give the names of variables to set, and
optionally values to which they should be set. These take the form of

varname

, or

!

varname

, or

varname

=

value

In the first form, the value will be set to \"1\". The second will
remove the given variable if already defined, and the third will set the
variable to the literal value given by

value

. Since version 2.0.51, Apache httpd will recognize occurrences of

\$1

..

\$9

within

value

and replace them by parenthesized subexpressions of

regex

.

\$0

provides access to the whole string matched by that pattern.

If the special value

\--early

is specified, and this directive is not in directory context, the
directive will be evaluated earlier during request processing. This
allows the output variables set by this directive to be used as input to
the \"early mode\" option of the

RequestHeader

directive. Available in 2.5.1 and later.

SetEnvIf Request\_URI \"\\.gif\$\" object\_is\_image=gif SetEnvIf
Request\_URI \"\\.jpg\$\" object\_is\_image=jpg SetEnvIf Request\_URI
\"\\.xbm\$\" object\_is\_image=xbm SetEnvIf Referer
www\\.mydomain\\.example\\.com intra\_site\_referral SetEnvIf
object\_is\_image xbm XBIT\_PROCESSING=1 SetEnvIf Request\_URI
\"\\.(.\*)\$\" EXTENSION=\$1 SetEnvIf \^TS \^\[a-z\] HAVE\_TS

The first three will set the environment variable

object\_is\_image

if the request was for an image file, and the fourth sets

intra\_site\_referral

if the referring page was somewhere on the

www.mydomain.example.com

Web site.

The last example will set environment variable

HAVE\_TS

if the request contains any headers that begin with \"TS\" whose values
begins with any character in the set \[a-z\].

Environment Variables in Apache HTTP Server

, for additional examples.

SetEnvIfExpr

Sets environment variables based on an ap\_expr expression

SetEnvIfExpr

expr \[!\]env-variable

\[=

value

\] \[\[!\]

env-variable

\[=

value

\]\] \...

server config

virtual host

directory

.htaccess

FileInfo

The

SetEnvIfExpr

directive defines environment variables based on an

If

ap\_expr

. These expressions will be evaluated at runtime, and applied

env-variable

in the same fashion as

SetEnvIf

.

SetEnvIfExpr \"tolower(req(\'X-Sendfile\')) ==
\'d:\\images\\very\_big.iso\')\" iso\_delivered

This would set the environment variable

iso\_delivered

every time our application attempts to send it via

X-Sendfile

A more useful example would be to set the variable rfc1918 if the remote
IP address is a private address according to RFC 1918:

SetEnvIfExpr \"-R \'10.0.0.0/8\' \|\| -R \'172.16.0.0/12\' \|\| -R
\'192.168.0.0/16\'\" rfc1918

Expressions in Apache HTTP Server

, for a complete reference and more examples.

If

can be used to achieve similar results.

mod\_filter

SetEnvIfNoCase

Sets environment variables based on attributes of the request without
respect to case

SetEnvIfNoCase

attribute regex \[!\]env-variable

\[=

value

\] \[\[!\]

env-variable

\[=

value

\]\] \...

server config

virtual host

directory

.htaccess

FileInfo

The

SetEnvIfNoCase

is semantically identical to the

SetEnvIf

directive, and differs only in that the regular expression matching is
performed in a case-insensitive manner. For example:

SetEnvIfNoCase Host Example\\.Org site=example

This will cause the

site

environment variable to be set to \"

example

\" if the HTTP request header field

Host:

was included and contained

Example.Org

,

example.org

, or any other combination.
